/*
 * Copyright (C) 2025 Aleksander <alekswilc> Wilczyński (aleks@wilczynski.me)
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * See LICENSE for more.
*/

import { ActivityType, ApplicationCommandType, ApplicationIntegrationType, Attachment, AttachmentBuilder, Client, ContextMenuCommandBuilder, Events, Interaction, InteractionContextType, MessageFlags, REST, Routes, SlashCommandBuilder } from 'discord.js';
const client = new Client({ intents: 53608447 });

client.on(Events.ClientReady, async () => {
    await client.user?.setStatus('idle');
    client.user?.setActivity({
        type: ActivityType.Listening,
        name: 'wilczynski.me'
    });
});

client.on(Events.InteractionCreate, async (interaction: Interaction) => {
    if (!interaction.isMessageContextMenuCommand()) return;

    switch (interaction.commandName) {
        case 'GIF': {
            if (!interaction.targetMessage.attachments.first())
                return interaction.reply({ content: 'No attachments', flags: [ MessageFlags.Ephemeral ] });

            const attachments = interaction.targetMessage.attachments.map(x => x.url);

            const buffers = await Promise.all((await Promise.all(attachments.map((x) => 
                fetch(x).then(x => x.arrayBuffer()).catch(() => null)
            ))).filter(x => !!x));

            interaction.reply({
                content: 'I turned your photos into GIFs ✨',
                files: buffers.map(x => new AttachmentBuilder(Buffer.from(x)).setName('file.gif'))
            });

            break;
        }
    }
});

(async () => {
    const rest = new REST().setToken(process.env.TOKEN!);

    await client.login(process.env.TOKEN!);
    console.log(`Logged as ${client.user?.username}`);

    await rest.put(
        Routes.applicationCommands(client.user?.id!),
        { body: [
            new ContextMenuCommandBuilder()
                .setName('GIF')
                .setType(ApplicationCommandType.Message)
                .setContexts([ InteractionContextType.Guild, InteractionContextType.BotDM, InteractionContextType.PrivateChannel ])
                .setIntegrationTypes([ ApplicationIntegrationType.UserInstall, ApplicationIntegrationType.GuildInstall ])
        ] },
    );

})();